class Base(Exception):
    """ghc base exception class"""


class TemplateNotFoundError(Base):
    """Raised when the specified template is not found"""


class GitHubRequestError(Base):
    """Raised when GitHub responses status code is not 200"""


class GitHubTokenUnsetError(Base):
    """Raised when GitHub personal access token is not set"""

    msg = 'GitHub Personal Access Token is not set.\n' \
        + 'Please specify --token option or ' \
        + 'set environment variable "GHC_TOKEN"'

    def __init__(self) -> None:
        super().__init__(self.msg)


class ReportError(Base):
    """Raised when fails to output the results"""



import argparse
from dataclasses import dataclass
from typing import Any, List, Optional

from ghc import __version__
from ghc.exceptions import GitHubTokenUnsetError
from ghc.utils import get_env


@dataclass
class CliOption:
    owner: str

    fmt: str = 'json'
    token: Optional[str] = None
    topics: Optional[List[str]] = None
    filename: Optional[str] = None

    def __post_init__(self) -> None:
        self.fmt = self.fmt.lower()

        if self.token is None:
            self.token = get_env('GHC_TOKEN', ignore_error=True)

        self.exists_token()

    def exists_token(self) -> None:
        if not self.token:
            raise GitHubTokenUnsetError()


def get_parser() -> argparse.ArgumentParser:
    return argparse.ArgumentParser(
        prog='ghc',
        allow_abbrev=False,
        description='List up GitHub user / org repositories filtered by topics'
    )


def setup_options(parser: argparse.ArgumentParser) -> None:
    parser.add_argument(
        'owner',
        help='Repository user or organization name to search'
    )

    parser.add_argument(
        '--token',
        help='''
        Personal Access Token to access repositories.
        Use the environment variable "GHC_TOKEN" instead.
        '''
    )

    parser.add_argument(
        '-t', '--topics',
        nargs='*',
        help='Filter repository using topics'
    )

    parser.add_argument(
        '-f', '--format',
        dest='fmt',
        default='json',
        choices=['json', 'md'],
        help='Format the results with json or md (markdown). Default is json'
    )

    parser.add_argument(
        '-o', '--output',
        dest='filename',
        help='Filename to save the results. Output to stdout if not specified'
    )

    parser.add_argument(
        '-V', '--version',
        action='version',
        version='%(prog)s {}'.format(__version__),
        help='Show command version'
    )


def parse_options(args: Any = None) -> CliOption:
    parser = get_parser()
    setup_options(parser)
    options = vars(parser.parse_args(args))
    return CliOption(**options)



import sys

from ghc.cli import parse_options
from ghc.exceptions import (
    GitHubRequestError, GitHubTokenUnsetError, ReportError,
    TemplateNotFoundError
)
from ghc.github import search_repositories_by_topics
from ghc.logger import get_logger
from ghc.reporter import report


logger = get_logger(__name__)


def main() -> None:
    try:
        options = parse_options()

        result = search_repositories_by_topics(
            options.token, owner=options.owner, topics=options.topics)
        logger.info('Succeeded to request to GitHub')

        report(result, fmt=options.fmt, filename=options.filename)
        logger.info('Completed report!!')
    except KeyboardInterrupt:
        logger.error('Abort')
        sys.exit(1)
    except GitHubTokenUnsetError as err:
        logger.error(err)
        sys.exit(1)
    except GitHubRequestError as err:
        logger.error(err)
        logger.error('Failed to request to GitHub')
        sys.exit(1)
    except (TemplateNotFoundError, ReportError) as err:
        logger.error(err)
        logger.error('Failed to output the result')
        sys.exit(1)
    except Exception as err:
        logger.error(err)
        sys.exit(1)


if __name__ == '__main__':
    main()



import json
from dataclasses import InitVar, dataclass, field
from pathlib import Path
from typing import IO, Any, Optional

from jinja2 import Environment, FileSystemLoader, Template

from ghc.exceptions import ReportError, TemplateNotFoundError
from ghc.stream import stream


@dataclass
class Reporter:
    def report(self, fd: IO[str], record: Any) -> None:
        raise NotImplementedError()


@dataclass
class JsonReporter(Reporter):
    indent: int = 2
    ensure_ascii: bool = False
    allow_nan: bool = True
    sort_keys: bool = True

    def report(self, fd: IO[str], record: Any) -> None:
        json.dump(
            record, fd, indent=self.indent,
            ensure_ascii=self.ensure_ascii, allow_nan=self.allow_nan,
            sort_keys=self.sort_keys)


BASE_DIR = Path(__file__).parent
TEMPLATE_DIR = f'{BASE_DIR}/templates'


@dataclass
class MrkdwnReporter(Reporter):
    tpl_path: InitVar[Optional[str]] = None

    search_path: str = field(init=False, default=TEMPLATE_DIR)
    template_name: str = field(init=False, default='default.j2')

    def __post_init__(self, tpl_path: Optional[str] = None) -> None:
        if tpl_path:
            p = Path(tpl_path)
            self.exists(p)
            self.search_path = str(p.parent)
            self.template_name = p.name

    @staticmethod
    def exists(tpl_path: Path) -> None:
        if tpl_path.exists() is False:
            raise TemplateNotFoundError(f'{tpl_path!r} is not found')

    def load_template(self) -> Template:
        env = Environment(
            loader=FileSystemLoader(self.search_path),
            autoescape=True,
            keep_trailing_newline=True)
        return env.get_template(self.template_name)

    def report(self, fd: IO[str], record: Any) -> None:
        tpl = self.load_template()
        tpl.stream(record).dump(fd)


REPORTER_MAP = {
    'json': JsonReporter,
    'md': MrkdwnReporter
}


def report(record: Any, fmt: str, filename: Optional[str] = None) -> None:
    try:
        with stream(filename) as fd:
            reporter = REPORTER_MAP[fmt]()
            reporter.report(fd, record)
    except TemplateNotFoundError:
        raise
    except Exception as err:
        raise ReportError(err)



import os
from typing import Any


def get_env(
        name: str,
        *,
        ignore_error: bool = False,
        default: Any = None) -> Any:
    try:
        os.environ[name]
    except KeyError as err:
        if ignore_error is False:
            raise err
        return default



from dataclasses import dataclass
from typing import ClassVar, List, Optional


@dataclass
class LanguageLogo:
    url: ClassVar[str] = 'https://cdn.jsdelivr.net/npm/' \
        + 'programming-languages-logos/src/{lang}/{lang}_24x24.png'

    supported_lang: ClassVar[List[str]] = [
        'c', 'csharp', 'cpp', 'css', 'go', 'haskell',
        'html', 'java', 'javascript', 'kotlin', 'lua',
        'php', 'python', 'r', 'ruby', 'swift', 'typescript'
    ]

    @classmethod
    def from_lang(cls, lang: str) -> Optional[str]:
        lang = lang.lower()
        if lang in cls.supported_lang:
            return cls.url.format(lang=lang)
        return None



from dataclasses import InitVar, asdict, dataclass, field
from operator import itemgetter
from typing import Any, ClassVar, Dict, List, Optional, Tuple, Union

import requests

from ghc.exceptions import GitHubRequestError
from ghc.logo import LanguageLogo


@dataclass
class Repository:
    isArchived: InitVar[bool]
    isTemplate: InitVar[bool]
    primaryLanguage: InitVar[Optional[Dict[str, str]]]

    name: str
    url: str
    description: Optional[str] = ''
    language: Optional[str] = field(init=False, default=None)
    language_logo_url: Optional[str] = field(init=False, default=None)
    is_archive: bool = field(init=False)
    is_template: bool = field(init=False)

    def __post_init__(self, isArchived, isTemplate, primaryLanguage) -> None:
        self.is_archive = isArchived
        self.is_template = isTemplate

        if primaryLanguage and (lang := primaryLanguage.get('name')):
            self.language = lang
            self.language_logo_url = LanguageLogo.from_lang(lang)

    def to_dict(self) -> Dict[str, Union[str, bool]]:
        return asdict(self)


@dataclass
class SearchQueryString:
    delimiter: ClassVar[str] = ' '
    q_user: ClassVar[str] = 'user:{user}'
    q_topic: ClassVar[str] = 'topic:{topic}'

    owner: str
    topics: Optional[List[str]] = None

    def to_string(self) -> str:
        query = self.q_user.format(user=self.owner)

        if self.topics:
            topics_str = [self.q_topic.format(topic=t) for t in self.topics]
            query += f'{self.delimiter}{self.delimiter.join(topics_str)}'

        return query


@dataclass
class GitHub:
    url: ClassVar[str] = 'https://api.github.com/graphql'

    token: Optional[str] = None

    def build_headers(self) -> Dict[str, str]:
        headers = {}

        if self.token:
            headers['Authorization'] = f'token {self.token}'

        return headers

    def request(
            self,
            query: Dict[str, Any],
            timeout: Union[int, Tuple[float, float]] = (10, 60),) -> Any:
        headers = self.build_headers()
        res = requests.post(
            self.url, headers=headers,
            json=query, timeout=timeout)

        if res.status_code != 200:
            res.raise_for_status()

        return res.json()


SEARCH_QUERY = '''
query {
    search(type: REPOSITORY, first: 100, query: "%s") {
        repositoryCount
        edges {
            node {
                ... on Repository {
                    name
                    description
                    isArchived
                    isTemplate
                    url
                    primaryLanguage {
                        name
                    }
                }
            }
        }
    }
}
'''


def search_repositories_by_topics(
        token: Optional[str] = None,
        **kwargs: Any) -> Dict[str, Any]:
    client = GitHub(token=token)
    search_q_str = SearchQueryString(**kwargs).to_string()
    query = SEARCH_QUERY % search_q_str

    try:
        res = client.request(query={'query': query})
        search_result = res['data']['search']
        result = {'count': search_result['repositoryCount']}
        tmp = [
            Repository(**edge['node']).to_dict()
            for edge in search_result['edges']]
        result['repositories'] = sorted(tmp, key=itemgetter('name'))
        return result
    except KeyError:
        raise GitHubRequestError(f'Unexpected response: {res}')
    except Exception as err:
        raise GitHubRequestError(err)



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ghc']

package_data = \
{'': ['*'], 'ghc': ['templates/*']}

install_requires = \
['Jinja2>=2.11.2,<3.0.0', 'requests>=2.24.0,<3.0.0']

entry_points = \
{'console_scripts': ['ghc = ghc.ghc:main']}

setup_kwargs = {
    'name': 'ghc',
    'version': '0.1.7',
    'description': 'List up GitHub user / org repositories filtered by topics (ghc = GitHub Collector)',
    'long_description': '# ghc (GitHub Collector)\n\n![PyPI](https://img.shields.io/pypi/v/ghc)\n![PyPI - Python Version](https://img.shields.io/pypi/pyversions/ghc)\n![GitHub](https://img.shields.io/github/license/homoluctus/ghc)\n\nList up GitHub user / org repositories filtered by topics\n\n<!-- TOC depthFrom:2 -->\n\n- [Feature](#feature)\n- [Installtion](#installtion)\n- [Usage](#usage)\n- [Examples](#examples)\n  - [Output JSON](#output-json)\n  - [Output Markdown](#output-markdown)\n  - [Use GHC_TOKEN instead of --token option](#use-ghc_token-instead-of---token-option)\n- [Roadmap](#roadmap)\n\n<!-- /TOC -->\n\n## Feature\n\n- List up GitHub org repositories\n  - filtered by topics\n- Support several formats\n  - JSON\n  - Markdown\n- Output the results to stdout or file\n  - Specify \'ghc/result.md\' as output filename and if \'ghc\' does not exist, create new directory\n\n## Installtion\n\n```bash\npip install ghc\n```\n\n## Usage\n\n```\nusage: ghc [-h] [--token TOKEN] [-t [TOPICS [TOPICS ...]]] [-f {json,md}] [-o FILENAME] [-V] owner\n\nList up GitHub user / org repositories filtered by topics\n\npositional arguments:\n  owner                 Repository user or organization name to search\n\noptional arguments:\n  -h, --help            show this help message and exit\n  --token TOKEN         Personal Access Token to access the private repository. Use the environment variable "GHC_TOKEN" instead.\n  -t [TOPICS [TOPICS ...]], --topics [TOPICS [TOPICS ...]]\n                        Filter repository using topics\n  -f {json,md}, --format {json,md}\n                        Format the results with json or md (markdown). Default is json\n  -o FILENAME, --output FILENAME\n                        Filename to output the results. Output stdout if not specified\n  -V, --version         Show command version\n```\n\n## Examples\n\n### Output JSON\n\n```bash\nghc homoluctus --token xxxxxxxx -f json -t python\n```\n\n<details>\n<summary>Result</summary>\n\n```json\n{\n  "count": 11,\n  "repositories": [\n    {\n      "description": null,\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "algorithms",\n      "url": "https://github.com/homoluctus/algorithms"\n    },\n    {\n      "description": "Sample for logging decorator",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "decolog",\n      "url": "https://github.com/homoluctus/decolog"\n    },\n    {\n      "description": "This tool helps to migrate DynamoDB to MySQL",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "dymy",\n      "url": "https://github.com/homoluctus/dymy"\n    },\n    {\n      "description": "Scan the vulnerability of Docker images stored in ECR",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "ecranner",\n      "url": "https://github.com/homoluctus/ecranner"\n    },\n    {\n      "description": "List up GitHub user / org repositories filtered by topics (ghc = GitHub Collector)",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "ghc",\n      "url": "https://github.com/homoluctus/ghc"\n    },\n    {\n      "description": "Python logging outputs as JSON",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "json-pyformatter",\n      "url": "https://github.com/homoluctus/json-pyformatter"\n    },\n    {\n      "description": "Analysis tool for Postfix log in /var/log/maillog",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "maillogger",\n      "url": "https://github.com/homoluctus/maillogger"\n    },\n    {\n      "description": "Notify today\'s wether information",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "ohtenki",\n      "url": "https://github.com/homoluctus/ohtenki"\n    },\n    {\n      "description": "Audit action for python dependencies (requirements.txt, Pipfile and poetry.lock)",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Shell",\n      "language_logo_url": null,\n      "name": "pip-audit-action",\n      "url": "https://github.com/homoluctus/pip-audit-action"\n    },\n    {\n      "description": "The CLI tool to query AWS CloudWatch Logs Insights :mag:",\n      "is_archive": false,\n      "is_template": false,\n      "language": "Python",\n      "language_logo_url": "https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png",\n      "name": "pyinsights",\n      "url": "https://github.com/homoluctus/pyinsights"\n    },\n    {\n      "description": "GitHub template for Python3.8",\n      "is_archive": false,\n      "is_template": true,\n      "language": "Makefile",\n      "language_logo_url": null,\n      "name": "python-template",\n      "url": "https://github.com/homoluctus/python-template"\n    }\n  ]\n}\n```\n\n</details>\n\n### Output Markdown\n\n\n```bash\nnghc homoluctus --token xxxxxxx -f md -t python\n```\n\n<details>\n<summary>Result (Raw)</summary>\n\n```markdown\n# Repositories\n\nTotal Count: 11\n\n|Name|URL|Language|Description|\n|:--:|:--:|:--:|:--|\n|algorithms|https://github.com/homoluctus/algorithms|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|None|\n|decolog|https://github.com/homoluctus/decolog|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Sample for logging decorator|\n|dymy|https://github.com/homoluctus/dymy|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|This tool helps to migrate DynamoDB to MySQL|\n|ecranner|https://github.com/homoluctus/ecranner|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Scan the vulnerability of Docker images stored in ECR|\n|ghc|https://github.com/homoluctus/ghc|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|List up GitHub user / org repositories filtered by topics (ghc = GitHub Collector)|\n|json-pyformatter|https://github.com/homoluctus/json-pyformatter|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Python logging outputs as JSON|\n|maillogger|https://github.com/homoluctus/maillogger|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Analysis tool for Postfix log in /var/log/maillog|\n|ohtenki|https://github.com/homoluctus/ohtenki|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Notify today&#39;s wether information|\n|pip-audit-action|https://github.com/homoluctus/pip-audit-action|Shell|Audit action for python dependencies (requirements.txt, Pipfile and poetry.lock)|\n|pyinsights|https://github.com/homoluctus/pyinsights|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|The CLI tool to query AWS CloudWatch Logs Insights :mag:|\n|python-template ![template](https://img.shields.io/badge/template-green.svg)|https://github.com/homoluctus/python-template|Makefile|GitHub template for Python3.8|\n\n> Generated by [ghc](https://github.com/homoluctus/ghc)\n\n```\n\n</details>\n\n<details>\n<summary>Result</summary>\n\n# Repositories\n\nTotal Count: 11\n\n|Name|URL|Language|Description|\n|:--:|:--:|:--:|:--|\n|algorithms|https://github.com/homoluctus/algorithms|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|None|\n|decolog|https://github.com/homoluctus/decolog|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Sample for logging decorator|\n|dymy|https://github.com/homoluctus/dymy|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|This tool helps to migrate DynamoDB to MySQL|\n|ecranner|https://github.com/homoluctus/ecranner|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Scan the vulnerability of Docker images stored in ECR|\n|ghc|https://github.com/homoluctus/ghc|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|List up GitHub user / org repositories filtered by topics (ghc = GitHub Collector)|\n|json-pyformatter|https://github.com/homoluctus/json-pyformatter|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Python logging outputs as JSON|\n|maillogger|https://github.com/homoluctus/maillogger|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Analysis tool for Postfix log in /var/log/maillog|\n|ohtenki|https://github.com/homoluctus/ohtenki|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|Notify today&#39;s wether information|\n|pip-audit-action|https://github.com/homoluctus/pip-audit-action|Shell|Audit action for python dependencies (requirements.txt, Pipfile and poetry.lock)|\n|pyinsights|https://github.com/homoluctus/pyinsights|![Python](https://cdn.jsdelivr.net/npm/programming-languages-logos/src/python/python_24x24.png)|The CLI tool to query AWS CloudWatch Logs Insights :mag:|\n|python-template ![template](https://img.shields.io/badge/template-green.svg)|https://github.com/homoluctus/python-template|Makefile|GitHub template for Python3.8|\n\n> Generated by [ghc](https://github.com/homoluctus/ghc)\n\n</details>\n\n### Use GHC_TOKEN instead of --token option\n\n```bash\nGHC_TOKEN=xxxxxxxx ghc homoluctus -f json -t python aws\n```\n\n## Roadmap\n\n- [ ] Ignore filter\n- [ ] Output to user-defined template\n',
    'author': 'homoluctus',
    'author_email': 'w.slife18sy@gmail.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/homoluctus/ghc',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



import logging
from typing import Union


def get_logger(
        name: str = __name__,
        level: Union[str, int] = logging.INFO) -> logging.Logger:
    """Get logger

    Args:
        name (str, optional): Defaults to __name__.
        level (Union[str, int], optional): Defaults to logging.INFO.

    Returns:
        logging.Logger:
    """

    logger = logging.getLogger(name)
    logger.setLevel(logging.INFO)
    return logger



import os
import sys
from dataclasses import dataclass, field
from typing import IO, Optional, Union


@dataclass
class StreamHandler:
    stream: IO[str] = sys.stdout

    def __enter__(self) -> IO[str]:
        return self._open()

    def __exit__(self, exc_type, exc_value, traceback) -> None:
        self.close()

    def _open(self) -> IO[str]:
        return self.stream

    def close(self) -> None:
        pass


@dataclass
class FileHandler(StreamHandler):
    filename: str

    mode: str = 'wt'
    encoding: str = 'utf-8'
    newline: Optional[str] = None

    stream: Optional[IO[str]] = field(init=False, default=None)  # type: ignore

    def __post_init__(self) -> None:
        self.mkdir()

    def mkdir(self) -> None:
        fp = self.filename.split('/')
        if len(fp) > 1:
            os.makedirs('/'.join(fp[:-1]), mode=0o755, exist_ok=True)

    def _open(self) -> IO[str]:
        if self.stream is None:
            self.stream = open(
                self.filename, mode=self.mode,
                newline=self.newline, encoding=self.encoding)
        return self.stream

    def close(self) -> None:
        if self.stream:
            self.stream.close()


def stream(
        filename: Optional[str] = None) -> Union[FileHandler, StreamHandler]:
    if filename:
        return FileHandler(filename)
    return StreamHandler()
